package de.twenty11.skysail.common.config.configadmin;

import java.util.Dictionary;
import java.util.HashMap;
import java.util.Map;

import org.apache.commons.lang.Validate;
import org.osgi.service.cm.ConfigurationException;
import org.osgi.service.cm.ManagedService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import de.twenty11.skysail.common.config.ConfigurationProvider;

/**
 * A {@link ConfigurationProvider} utilizing the OSGi Configuration Admin service.
 * 
 * The file containing the configuration must have the name defined as service.pid in configurationProvider.xml.
 * 
 * @see ConfigurationProvider for more information.
 * 
 */
public class ConfigAdminConfigurationProvider implements ConfigurationProvider, ManagedService {

    private static Logger logger = LoggerFactory.getLogger(ConfigAdminConfigurationProvider.class);

    private Map<String, String> configuration = new HashMap<String, String>();

    @Override
    public String getName() {
        return this.getClass().getSimpleName();
    }

    @Override
    public String getConfigForKey(String key) {
        Validate.notNull(key, "key may not be null");
        Validate.isTrue(!key.contains(" "), "key may not contain whitespace");
        return configuration.get(key);
    }

    @Override
    public void updated(@SuppressWarnings("rawtypes") Dictionary properties) throws ConfigurationException {
        logger.info("updating configuration provider '{}', setting values for:", getName());
        while (properties != null && properties.keys().hasMoreElements()) {
            Object key = properties.keys().nextElement();
            logger.info(" > " + key.toString() + ",");
            configuration.put((String) key, (String) properties.get(key));
            properties.remove(key);
        }
    }

}
